/*******************************************************************************

 WINBINDER - A native Windows binding for PHP

 Copyright  2004 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 ZEND wrapper for low-level functions

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "phpwb.h"

//-------------------------------------------------------------------- VARIABLES

//----------------------------------------------------------- EXPORTED FUNCTIONS

ZEND_FUNCTION(wb_send_message)
{
    long msg, w, l;
	long pwbo;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "llll", &pwbo, &msg, &w, &l) == FAILURE)
		return;

	RETURN_LONG(wbSendMessage((PWBOBJ)pwbo, (UINT)msg, (WPARAM)w, (LPARAM)l));
}

/* Get the contents of a memory area */

ZEND_FUNCTION(wb_peek)
{
	long address, bytes = 0;
	char *ptr;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "l|l", &address, &bytes) == FAILURE)
		return;

	if(!address)
		RETURN_NULL();

	bytes = max(0, bytes);
	ptr = (char *)address;
	if(bytes == 0) {							// Want a zero-terminated string?
		if(!IsBadStringPtr(ptr, 32767))
			RETURN_STRINGL(ptr, strlen(ptr), TRUE)
	} else {									// No, want a memory dump
		if(!IsBadReadPtr(ptr, bytes))
			RETURN_STRINGL(ptr, bytes, TRUE)
	}
	zend_error(E_WARNING, "Cannot read from location %d", (int)ptr);
	RETURN_NULL();
}

/* Change the contents of a memory area */

ZEND_FUNCTION(wb_poke)
{
	long address, bytes;
	char *contents;
	int len;
	void *ptr;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "ls|l", &address, &contents, &len, &bytes) == FAILURE)
		return;

	if(!address)
		RETURN_NULL();
	if(!contents)
		RETURN_NULL();
	if(!len)
		RETURN_NULL();
	if(bytes == 0)
		RETURN_NULL();

	bytes = max(0, bytes);
	ptr = (void *)address;

	if(IsBadWritePtr(ptr, bytes)) {
		zend_error(E_WARNING, "Cannot write to location %d", (int)ptr);
		RETURN_NULL();
	}

	if(bytes != 0)							// Is it a zero-terminated string?
		strcpy(ptr, contents);
	else
		memcpy(ptr, contents, bytes);

	RETURN_TRUE;
}

/*

Get the address of a string
// TODO: get the address of other objects

*/

ZEND_FUNCTION(wb_get_address)
{
	char *s;
	int s_len;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "s", &s, &s_len) == FAILURE)
		return;

	RETURN_LONG((LONG)(void *)s)
}

//------------------------------------------------------------------ END OF FILE
